/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>
#include <fstream>
#include <vector>
#include <o2scl/sn_fermion.h>
#include <o2scl/eff_fermion.h>
#include <o2scl/rel_fermion.h>
#include <o2scl/sn_classical.h>
#include <o2scl/classical.h>
#include <o2scl/nonrel_fermion.h>

/*
  Todo: 
  - Test with inc_rest_mass = true and inc_rest_mass = false
  - Compute average and worst-case performance
  - Test different values of the temperature
*/

using namespace std;
using namespace o2scl;

typedef struct {
  double lmot;
  double lpsi;
  double n;
  double ed;
  double pr;
  double en1;
  double en2;
  double err;
} pm_entry;

class part_test {
public:

  sn_fermion sf;
  eff_fermion ef;
  rel_fermion rf;
  size_t n;

  std::vector<pm_entry> tab;

  part_test() {
    sf.init(0.0,2.0);
    ef.init(0.0,2.0);
    rf.init(0.0,2.0);
  }

  int read_file() {
    ifstream fin("../src/internal/pm.save");
    for(size_t i=0;i<35;i++) {
      pm_entry p;
      fin >> p.lmot >> p.lpsi >> p.n >> p.ed >> p.pr 
	  >> p.en1 >> p.en2 >> p.err;
      tab.push_back(p);
    }
    fin.close();
    n=tab.size();
    return 0;
  }

  int test() {
    for(size_t i=0;i<n;i++) {

      double T=1.0;

      // With inc_rest_mass true

      double m=pow(10.0,tab[i].lmot)*T;
      double mu=m+T*pow(10.0,tab[i].lpsi);
      
      cout << "T,m,mu: " << T << " " << m << " " << mu << endl;
      
      sf.m=m;
      sf.mu=mu;
      sf.calc_mu(T);
      cout << sf.n << " " << sf.ed << " " << sf.pr << " " << sf.ed << " "
	   << sf.en << endl;

      ef.m=m;
      ef.mu=mu;
      ef.calc_mu(T);
      cout << ef.n << " " << ef.ed << " " << ef.pr << " " << ef.ed << " "
	   << ef.en << endl;

      rf.m=m;
      rf.mu=mu;
      rf.calc_mu(T);
      cout << rf.n << " " << rf.ed << " " << rf.pr << " " << rf.ed << " "
	   << rf.en << endl;

      cout << tab[i].n << " " << tab[i].ed << " " << tab[i].pr << " " 
	   << tab[i].ed << " " << tab[i].en1 << endl;
      cout << endl;

    }
    return 0;
  }

};

int main(void) {

  cout.setf(ios::scientific);
  cout.setf(ios::showpos);

  part_test pt;
  pt.read_file();
  pt.test();

  return 0;
}

