/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

/* Example: ex_anneal.cpp
   -------------------------------------------------------------------
   An example to demonstrate minimization by simulated annealing
*/
  
#include <iostream>
#include <cmath>
#include <gsl/gsl_sf_bessel.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/multi_funct.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_anneal.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

// A simple function with many local minima. A "greedy" minimizer
// would likely fail to find the correct minimum.
double function(size_t nvar, const ovector_base &x, int &vp) {
  double a, b;
  a=(x[0]-2.0);
  b=(x[1]+3.0);
  return -gsl_sf_bessel_J0(a)*gsl_sf_bessel_J0(b);
}

int main(int argc, char *argv[]) {
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);

  gsl_anneal<int,multi_funct<int> > ga;
  double result;
  ovector init(2);
  
  multi_funct_fptr_noerr<int> fx(function);

  ga.ntrial=1000;
  ga.verbose=1;
  ga.tolx=1.0e-7;
  ga.T_dec=1.1;

  int vpx=0;

  // Choose an initial point at a local minimum away from
  // the global minimum
  init[0]=9.0;
  init[1]=9.0;
  
  // Perform the minimization
  ga.mmin(2,init,result,vpx,fx);
  cout << "x: " << init[0] << " " << init[1] 
       << ", minimum function value: " << result << endl;
  cout << endl;

  // Test that it found the global minimum
  t.test_rel(init[0],2.0,1.0e-2,"another test - value");
  t.test_rel(init[1],-3.0,1.0e-2,"another test - value 2");
  t.test_rel(result,-1.0,1.0e-2,"another test - min");

  t.report();
  
  return 0;
}
// End of example
