/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
/* Example: ex_cold_nstar.cpp
   -------------------------------------------------------------------
   This example solves the TOV equations using class cold_nstar using a
   relativistic mean-field EOS from class rmf_eos.
*/

#include <o2scl/collection.h>
#include <o2scl/text_file.h>
#include <o2scl/cold_nstar.h>
#include <o2scl/rmf_eos.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

// For hc_mev_fm
using namespace o2scl_const;

int main(void) {

  cout.setf(ios::scientific);

  test_mgr t;
  t.set_output_level(1);

  cold_nstar nst;

  // Initialize EOS
  rmf_eos rmf;

  rmf.load("NL3");
  
  rmf.saturation();
  cout << "Saturation density: " << rmf.n0 << endl;
  cout << "Binding energy: " << rmf.eoa*hc_mev_fm << endl;
  cout << "Effective mass: " << rmf.msom << endl;
  cout << "Symmetry energy: " << rmf.esym*hc_mev_fm << endl;
  cout << "Compressibility: " << rmf.comp*hc_mev_fm << endl;
  
  // Compute EOS, include muons
  nst.include_muons=true;
  nst.set_eos(rmf);
  nst.calc_eos();
  table &te=nst.get_eos_results();

  // Output EOS results to a file
  collection co;
  text_out_file *tof=new text_out_file("ex_cold_nstar.out");
  co.out_one(tof,"table","tov",&te);
  delete tof;

  // Compute mass vs. radius
  nst.calc_nstar();
  table &tr=nst.get_tov_results();
  cout << "Maximum mass: " << tr.max("gm") << endl;
  cout << "Radius of maximum mass star: " 
       << tr.get("r",tr.lookup("gm",tr.max("gm"))) << endl;
  cout << "Central baryon density of maximum mass star: ";
  cout << tr.get("nb",tr.lookup("gm",tr.max("gm"))) << endl;

  t.report();
  return 0;
}
// End of example
