/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
/* Example: ex_file.cpp
   -------------------------------------------------------------------
   This example demonstrates file I/O
   
   Note that the binary files automatically store the full precision
   for floating-point numbers, but that the text files do not. The
   precision for text files can be modified, see \ref text_out_file
   for details.
*/

#include <o2scl/collection.h>
#include <o2scl/file_detect.h>
#include <o2scl/base_ioc.h>

using namespace std;
using namespace o2scl;

int main(void) {
  cout.setf(ios::scientific);
  
  err_hnd=&alt_err_hnd;
  alt_err_hnd.set_mode(0);
  
  test_mgr t;
  t.set_output_level(2);

  /// Some data
  char c='t';
  double d=sqrt(2.0);
  int i=10;
  string s="string test";

  /// Make sure the I/O objects are present
  base_ioc bio;

  /// We create a collection, and add all of the data 
  /// items to the collection
  collection co;

  co.add_void("char_test","char",&c);
  co.add_void("double_test","double",&d);
  co.add_void("int_test","int",&i);
  co.add_void("string_test","string",&s);
  
  /// In order to output things to stdout later
  text_out_file tout(&cout);

  /// Output the data in the collection to several 
  /// different types of files
  
  co.fout("ex_file_1");
  
  text_out_file *tof2=new text_out_file("ex_file_2.gz");
  co.fout(tof2);
  delete tof2;

  binary_out_file *bof=new binary_out_file("ex_file_3");
  co.fout(bof);
  delete bof;

  binary_out_file *bof2=new binary_out_file("ex_file_4.gz");
  co.fout(bof2);
  delete bof2;

  /// Erase all of the information in the collection
  co.clear();
  
  /// Now read in each of the files

  file_detect fd;

  cout.precision(10);

  cout << "ex_file_1:\n" << endl;
  co.fin(fd.open("ex_file_1"));
  if (fd.is_compressed()) cout << "Compressed." << endl;
  if (fd.is_binary()) cout << "Binary." << endl;
  fd.close();
  co.fout(&tout);
  co.clear();

  cout << "\nex_file_2:\n" << endl;
  co.fin(fd.open("ex_file_2.gz"));
  if (fd.is_compressed()) cout << "Compressed." << endl;
  if (fd.is_binary()) cout << "Binary." << endl;
  fd.close();
  co.fout(&tout);
  co.clear();

  cout << "\nex_file_3:\n" << endl;
  co.fin(fd.open("ex_file_3"));
  if (fd.is_compressed()) cout << "Compressed." << endl;
  if (fd.is_binary()) cout << "Binary." << endl;
  fd.close();
  co.fout(&tout);
  co.clear();
  
  cout << "\nex_file_4:\n" << endl;
  co.fin(fd.open("ex_file_4.gz"));
  if (fd.is_compressed()) cout << "Compressed." << endl;
  if (fd.is_binary()) cout << "Binary." << endl;
  fd.close();
  co.fout(&tout);
  co.clear();

  t.report();
  return 0;
}
// End of example
