/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

/* Example: ex_mmin.cpp
   -------------------------------------------------------------------
   Example usage of the multidimensional minimizers
*/

#include <cmath>
#include <o2scl/test_mgr.h>
#include <o2scl/multi_funct.h>
#include <o2scl/gsl_mmin_simp2.h>
#include <o2scl/gsl_mmin_conf.h>
#include <o2scl/gsl_mmin_conp.h>
#include <o2scl/gsl_mmin_bfgs2.h>

using namespace std;
using namespace o2scl;

class cl {

public:

  int mfn(size_t nv, const ovector_base &x, double &y, int &pa) {
    y=(x[0]-2.0)*(x[0]-2.0)+(x[1]-1.0)*(x[1]-1.0);
    return 0;
  }
  
};

int main(void) {
  cl acl;
  ovector x(2);
  int vp=0;
  double fmin;
  test_mgr t;

  t.set_output_level(1);
  cout.setf(ios::scientific);

  /**
     Using a member function with \ref ovector objects
  */
  multi_funct_mfptr<cl,int,ovector_base> f1(&acl,&cl::mfn);
  gsl_mmin_simp2<int> gm1;
  gsl_mmin_conf<int> gm2;
  gsl_mmin_conp<int> gm3;
  gsl_mmin_bfgs2<int> gm4;
    
  x[0]=0.5;
  x[1]=0.5;
  gm1.mmin(2,x,fmin,vp,f1);
  cout << gm1.last_ntrial << endl;
  cout << "Found minimum at: " << x << endl;
  t.test_rel(x[0],2.0,1.0e-4,"1a");
  t.test_rel(x[1],1.0,1.0e-4,"1b");

  x[0]=0.5;
  x[1]=0.5;
  gm2.mmin(2,x,fmin,vp,f1);
  cout << gm2.last_ntrial << endl;
  cout << "Found minimum at: " << x << endl;
  t.test_rel(x[0],2.0,1.0e-4,"2a");
  t.test_rel(x[1],1.0,1.0e-4,"2b");

  x[0]=0.5;
  x[1]=0.5;
  gm3.mmin(2,x,fmin,vp,f1);
  cout << gm3.last_ntrial << endl;
  cout << "Found minimum at: " << x << endl;
  t.test_rel(x[0],2.0,1.0e-4,"3a");
  t.test_rel(x[1],1.0,1.0e-4,"3b");

  x[0]=0.5;
  x[1]=0.5;
  gm4.err_nonconv=false;
  gm4.mmin(2,x,fmin,vp,f1);
  cout << gm4.last_ntrial << endl;
  cout << "Found minimum at: " << x << endl;
  t.test_rel(x[0],2.0,1.0e-4,"4a");
  t.test_rel(x[1],1.0,1.0e-4,"4b");

  t.report();
  return 0;
}
// End of example
