/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
  
#include <cmath>
#include <cstdlib>
#include <gsl/gsl_siman.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/multi_funct.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_anneal.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

double funx(size_t nv, const ovector_base &x, int &vp) {
  double ret, a;
  a=x[0]-2.0;
  ret=-exp(-a*a);
  return ret;
}

double funz(size_t nvar, const double x[2], int &vp) {
  double ret, a, b;
  a=x[0]-2.0;
  b=x[1]-3.0;
  ret=a*a+b*b+a*b;
  return ret;
}

int main(int argc, char *argv[]) {
  test_mgr t;
  t.set_output_level(2);

  cout.setf(ios::scientific);
  
  gsl_anneal<int,multi_funct<int> > ga;
  double result;
  ovector init(2);
  double init2[2];
  
  multi_funct_fptr_noerr<int > fx(funx);

  /// 1d to vectors

  int vpx=0;

  init[0]=0.1;
  init[1]=0.2;
  ga.tolx=1.0e-6;
  ga.mmin(1,init,result,vpx,fx);
  cout << init[0] << " " << result << endl;
  t.test_rel(init[0],2.0,1.0e-3,"another test - value");
  t.test_rel(result,-1.0,1.0e-3,"another test - min");

  /// 2d with arrays
  
  gsl_anneal<int,multi_vfunct<int,2>,double [2],double[2],
    array_alloc<double[2]> > ga2;
  multi_vfunct_fptr_noerr<int,2> fz(funz);

  ga2.ntrial=300;

  init2[0]=0.0;
  init2[1]=0.0;
  ga2.mmin(2,init2,result,vpx,fz);
  cout << init2[0] << " " << init2[1] << " " << result << endl;
  t.test_rel(init2[0],2.0,1.0e-2,"2d test 1");
  t.test_rel(init2[1],3.0,1.0e-2,"2d test 2");

  // Test verbose=1

  init[0]=15.5;
  ga.verbose=1;
  ga.mmin(1,init,result,vpx,fx);
  
  t.report();
  
  return 0;
}

