/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/err_hnd.h>
#include <o2scl/exception.h>

using namespace std;
using namespace o2scl;

err_hnd_cpp o2scl::def_err_hnd;

err_hnd_cpp::err_hnd_cpp() {

#if O2SCL_USE_GSL_HANDLER
  err_hnd=&alt_err_hnd;
#else
  err_hnd=this;
#endif

  gsl_set_error_handler(err_hnd->gsl_hnd);

}

void err_hnd_cpp::set(const char *reason, const char *file, 
		      int line, int lerrno) {
  
  a_errno=lerrno;
  a_file=(char *)file;
  a_line=line;
  strncpy(a_reason,reason,200);
      
  if (lerrno==gsl_ememtype) {
    throw exc_logic_error(a_reason);
  } else if (lerrno==gsl_einval || lerrno==gsl_ebadtol ||
	     lerrno==gsl_ebadlen || lerrno==gsl_enotsqr ||
	     lerrno==gsl_eindex) {
    throw exc_invalid_argument(a_reason);
  } else if (lerrno==gsl_failure || lerrno==gsl_efailed ||
	     lerrno==gsl_esanity || lerrno==gsl_eunsup ||
	     lerrno==gsl_eunimpl) {
    throw exc_exception();
  } else if (lerrno==gsl_edom || lerrno==gsl_eundrflw ||
	     lerrno==gsl_erange) {
    throw exc_range_error(a_reason);
  } else if (lerrno==gsl_ezerodiv || lerrno==gsl_eovrflw) {
    throw exc_overflow_error(a_reason);
  } else if (lerrno==gsl_eof || lerrno==gsl_efilenotfound) {
    throw exc_ios_failure(a_reason);
  } else {
    throw exc_runtime_error(a_reason);
  }
      
  return;
}

