/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/file_detect.h>
#include <cstdlib>

using namespace std;
using namespace o2scl;

file_detect::file_detect() {
  iffp=0;
}

in_file_format *file_detect::open(const char *s, bool err_on_fail) {
  string cmd;
  ofstream *ofsp;

  user_filename=s;
  int len=user_filename.length(), ret;
  
  if (len>3 && user_filename[len-1]=='z' && user_filename[len-2]=='g' &&
      user_filename[len-3]=='.') {

    int ir=o2scl_make_tfile(temp_filename);

    cmd=((string)"cp ")+user_filename+" "+temp_filename+".gz";
    ret=system(cmd.c_str());
    if (ret!=0) {
      if (err_on_fail) {
	O2SCL_ERR2("Copying compressed file to temporary ",
		   "directory failed in file_detect::open().",gsl_efailed);
      }
      return 0;
    }
    
    cmd=((string)"gunzip -f ")+temp_filename+".gz";
    ret=system(cmd.c_str());
    if (ret!=0) {
      if (err_on_fail) {
	O2SCL_ERR("Decompression failed in file_detect::open().",
		  gsl_efailed);
      }
      return 0;
    }

    compressed=true;

  } else if (len>4 && user_filename[len-1]=='2' && 
	     user_filename[len-2]=='z' && user_filename[len-3]=='b' && 
	     user_filename[len-4]=='.') {

    int ir=o2scl_make_tfile(temp_filename);

    cmd=((string)"cp ")+user_filename+" "+temp_filename+".bz2";
    ret=system(cmd.c_str());
    if (ret!=0) {
      if (err_on_fail) {
	O2SCL_ERR2("Copying compressed file to temporary ",
		   "directory failed in file_detect::open().",gsl_efailed);
      }
      return 0;
    }
    
    cmd=((string)"bunzip2 -f ")+temp_filename+".bz2";
    ret=system(cmd.c_str());
    if (ret!=0) {
      if (err_on_fail) {
	O2SCL_ERR("Decompression failed in file_detect::open().",
		  gsl_efailed);
      }
      return 0;
    }

    compressed=true;

  } else {

    temp_filename=user_filename;

    compressed=false;
  }

  std::ifstream *tmps=new ifstream;
  char ch;
  tmps->open(temp_filename.c_str(),ios::binary);
  if (!(*tmps)) {
    if (err_on_fail) {
      O2SCL_ERR("Failed to open file in file_detect::open().",
		gsl_efilenotfound);
    }
    delete tmps;
    tmps=0;
    return 0;
  }
  if (!(tmps->read(&ch,1))) {
    if (err_on_fail) {
      O2SCL_ERR("Failed to read type in file_detect::open().",
		gsl_efailed);
    }
    delete tmps;
    tmps=0;
    return 0;
  }
  tmps->close();
  delete tmps;
  if (ch==1) {
    binary=true;
    iffp=(in_file_format *)(new binary_in_file(temp_filename));
  } else {
    binary=false;
    iffp=(in_file_format *)(new text_in_file(temp_filename));
  }

  return iffp;
}

int file_detect::close() {

  if (iffp!=0) {

    if (binary==true) {
      binary_in_file *bif=(binary_in_file *)iffp;
      delete bif;
    } else {
      text_in_file *tif=(text_in_file *)iffp;
      delete tif;
    }

    if (compressed) {

      // Remove the temporary file created by decompression
      string cmd;
      cmd=((string)"rm -f ")+temp_filename;
      system(cmd.c_str());
    }

  }
  // Ensure that an extra call to close() doesn't cause problems
  iffp=0;
  return 0;
}

