/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_LIB_SETTINGS_H
#define O2SCL_LIB_SETTINGS_H
#include <iostream>
#include <string>

/** \file lib_settings.h
    \brief File for definitions for lib_settings_class
*/

/** 
    \brief The main \o2 namespace
    
    By default, all \o2 classes and functions which are not listed as
    being in one of \o2's smaller specialized namespaces are in this
    namespace. This namespace has been removed from the documentation
    to simplify the formatting.

    \htmlonly
    For a full list of all the 
    O<span style='position: relative; top: 0.3em; font-size: 0.8em'>2</span>scl 
    classes, see
    <a href="annotated.html">Data Structures</a>.

    For a full list of all the
    O<span style='position: relative; top: 0.3em; font-size: 0.8em'>2</span>scl 
    global objects which are not
    classes, see <a href="globals.html">Files::Globals</a>.
    
    \endhtmlonly
    
    This namespace documentation is in the file 
    <tt>src/base/lib_settings.h</tt>
*/
namespace o2scl {
}

/** \brief The namespace for linear algebra classes and functions

    This namespace documentation is in the file 
    <tt>src/base/lib_settings.h</tt>
*/
namespace o2scl_linalg {
}

/** \brief The namespace for linear algebra classes and functions 
    with operator()

    This namespace contains an identical copy of all the functions given 
    in the \ref o2scl_cblas namespace, but perform array indexing
    with \c operator() rather than \c operator[]. See \ref o2scl_linalg
    for the function listing and documentation.

    This namespace documentation is in the file 
    <tt>src/base/lib_settings.h</tt>
*/
namespace o2scl_linalg_paren {
}

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief A class to manage global library settings

      A global object of this type is defined in
      <tt>lib_settings.h</tt> named \ref lib_settings .
  */
  class lib_settings_class {

  public:

    lib_settings_class();

    ~lib_settings_class();

    /** \brief Return the data directory */
    std::string get_data_dir() {
      return data_dir;
    }

    /** \brief Set the data directory */
    int set_data_dir(std::string dir) {
      data_dir=dir;
      return 0;
    }
    
    /** \brief Return the temp file directory */
    std::string get_tmp_dir() {
      return tmp_dir;
    }

    /** \brief Set the temp file directory */
    int set_tmp_dir(std::string dir) {
      tmp_dir=dir;
      return 0;
    }

    /// Return 1 if the EOS library was installed
    int eos_installed();

    /// Return 1 if the particle library was installed
    int part_installed();

    /** 
	\brief Return true if range checking was turned on during 
	installation (default true)
    */
    bool range_check();

    /// Return the library version
    std::string o2scl_version();

  protected:

#ifndef DOXYGEN_INTERNAL

    /// The present data directory
    std::string data_dir;
    /// The present temp file directory
    std::string tmp_dir;

#endif
  
  };

  /** 
      \brief The global library settings object

      This global object is used by \ref polylog and some of the \o2e
      classes to find data files. It may also be used by the end-user
      to probe details of the \o2 installation.
   */
  extern lib_settings_class lib_settings;

#ifndef DOXYGENP
}
#endif

#endif
