/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/text_file.h>
#include <cstdlib>

using namespace std;
using namespace o2scl;

// ------------------------------------------------------------
// text_out_file functions
// ------------------------------------------------------------

text_out_file::text_out_file() {
  strout=0;
  outs=0;
}

text_out_file::text_out_file(ostream *out_file, int width,
			     bool bracket_objs) {

  extra_brackets=bracket_objs;

  strout=new std::ostringstream;
  outs=out_file;
  props=out_file;

  strout->precision(props->precision());
  strout->fill(props->fill());
  strout->flags(props->flags());
  
  file_width=width-1;
  from_string=false;
  
}

text_out_file::text_out_file(string file_name, ostream *prop,
			     bool append, int width,
			     bool bracket_objs) {
  extra_brackets=bracket_objs;

  ofstream *fout=new ofstream;
  outs=(ostream *)fout;
  
  int len=file_name.length();

  if (file_name[len-1]=='z' && file_name[len-2]=='g' &&
      file_name[len-3]=='.') {

    user_filename=file_name.substr(0,len-3);
    
    string stemplat;

    int ir=o2scl_make_tfile(stemplat);
    if (append) fout->open(stemplat.c_str(),ios::app);
    else fout->open(stemplat.c_str());

    temp_filename=stemplat;

    compressed=true;
    gzip=true;

  } else if (file_name[len-1]=='2' && file_name[len-2]=='z' &&
	     file_name[len-3]=='b' && file_name[len-4]=='.') {

    user_filename=file_name.substr(0,len-4);
    
    string stemplat;

    int ir=o2scl_make_tfile(stemplat);
    if (append) fout->open(stemplat.c_str(),ios::app);
    else fout->open(stemplat.c_str());

    temp_filename=stemplat;

    compressed=true;
    gzip=false;

  } else {

    if (append) fout->open(file_name.c_str(),ios::app);
    else fout->open(file_name.c_str());

    compressed=false;
  }

  strout=new std::ostringstream;
  
  if (prop==0) props=outs;
  else props=prop;

  strout->precision(props->precision());
  strout->fill(props->fill());
  strout->flags(props->flags());
  
  file_width=width-1;
  from_string=true;

}

int text_out_file::open(ostream *out_file, int width,
			bool bracket_objs) {

  if (strout!=0 || outs!=0) {
    O2SCL_ERR_RET("File already opened in text_out_file::open().",
		  gsl_efailed);
  }

  extra_brackets=bracket_objs;

  strout=new std::ostringstream;
  outs=out_file;
  props=out_file;
  
  strout->precision(props->precision());
  strout->fill(props->fill());
  strout->flags(props->flags());
  
  file_width=width-1;
  from_string=false;
  
  return gsl_success;
}

int text_out_file::open(string file_name, ostream *prop,
			bool append, int width,
			bool bracket_objs) {
  
  if (strout!=0 || outs!=0) {
    O2SCL_ERR_RET("File already opened in text_out_file::open().",
		  gsl_efailed);
  }

  extra_brackets=bracket_objs;

  ofstream *fout=new ofstream;
  outs=(ostream *)fout;
  
  int len=file_name.length();

  if (file_name[len-1]=='z' && file_name[len-2]=='g' &&
      file_name[len-3]=='.') {

    user_filename=file_name.substr(0,len-3);
    
    string stemplat;
    
    int ir=o2scl_make_tfile(stemplat);
    if (append) fout->open(stemplat.c_str(),ios::app);
    else fout->open(stemplat.c_str());

    temp_filename=stemplat;

    compressed=true;
    gzip=true;

  } else if (file_name[len-1]=='2' && file_name[len-2]=='z' &&
	     file_name[len-3]=='b' && file_name[len-4]=='.') {

    user_filename=file_name.substr(0,len-4);
    
    string stemplat;

    int ir=o2scl_make_tfile(stemplat);
    if (append) fout->open(stemplat.c_str(),ios::app);
    else fout->open(stemplat.c_str());

    temp_filename=stemplat;

    compressed=true;
    gzip=false;

  } else {

    if (append) fout->open(file_name.c_str(),ios::app);
    else fout->open(file_name.c_str());

    compressed=false;
  }

  strout=new std::ostringstream;
  
  if (prop==0) props=outs;
  else props=prop;

  strout->precision(props->precision());
  strout->fill(props->fill());
  strout->flags(props->flags());
  
  file_width=width-1;
  from_string=true;

  return gsl_success;
}

text_out_file::~text_out_file() {

  if (outs!=0 && strout!=0) {
    end_line();
  }
  
  if (from_string && outs!=0) {

    if (compressed) {
      string cmd;
      int ret;

      cmd=((string)"mv ")+temp_filename+" "+user_filename;
      ret=system(cmd.c_str());
      if (ret!=0) {
	delete ((ofstream *)outs);
	delete strout;
        O2SCL_ERR("Creating file failed. File closed anyway.",gsl_efailed);
        return;
      }
       
      if (gzip) cmd=((string)"gzip -f ")+user_filename;
      else cmd=((string)"bzip2 -f ")+user_filename;
      ret=system(cmd.c_str());
      if (ret!=0) {
	delete ((ofstream *)outs);
	delete strout;
        O2SCL_ERR("Compression failed. File closed anyway.",gsl_efailed);
        return;
      }

    }
    
    ofstream *fout=(ofstream *)outs;
    delete fout;
    outs=0;
  }

  if (strout!=0) {
    delete strout;
  }
}

int text_out_file::close() {

  if (outs!=0 && strout!=0) {
    end_line();
  }
  
  if (from_string && outs!=0) {

    if (compressed) {
      string cmd;
      int ret;

      cmd=((string)"mv ")+temp_filename+" "+user_filename;
      ret=system(cmd.c_str());
      if (ret!=0) {
	delete ((ofstream *)outs);
	delete strout;
        O2SCL_ERR("Creating file failed. File closed anyway.",gsl_efailed);
        return 0;
      }
       
      if (gzip) cmd=((string)"gzip -f ")+user_filename;
      else cmd=((string)"bzip2 -f ")+user_filename;
      ret=system(cmd.c_str());
      if (ret!=0) {
	delete ((ofstream *)outs);
	delete strout;
        O2SCL_ERR("Compression failed. File closed anyway.",gsl_efailed);
        return 0;
      }

    }
    
    ofstream *fout=(ofstream *)outs;
    delete fout;
    outs=0;
  }

  if (strout!=0) {
    delete strout;
    strout=0;
  }
  
  return 0;
}

int text_out_file::init_file() {
  return 0;
}

int text_out_file::end_line() {
  flush();
  string s=strout->str();
  if (s.length()>0) {
    (*outs) << s << endl;
  }
  
  // Delete the old ostrstream, and make a new ostrstream
  // and set the flags right.
  delete strout;
  strout=new ostringstream;

  strout->precision(props->precision());
  strout->fill(props->fill());
  strout->flags(props->flags());

  return 0;
}

int text_out_file::bool_out(bool dat, std::string name) {
  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::bool_out()",
		  gsl_efailed);
  }
  if (name.length()>0) {
    (*strout) << name << ' ' << dat << ' ';
  } else {
    (*strout) << dat << ' ';
  }
  flush();
  return 0;
}

int text_out_file::char_out(char dat, std::string name) {
  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::char_out()",
		  gsl_efailed);
  }
  if (extra_brackets && (dat=='<' || dat=='>')) {
    if (name.length()>0) {
      (*strout) << name << ' ' << dat << dat << ' ';
    } else {
      (*strout) << dat << dat << ' ';
    }
  } else {
    if (name.length()>0) {
      (*strout) << name << ' ' << dat << ' ';
    } else {
      (*strout) << dat << ' ';
    }
    flush();
  }
  return 0;
}

int text_out_file::char_out_internal(char dat, std::string name) {
  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::char_out_internal()",
		  gsl_efailed);
  }
  if (name.length()>0) {
    (*strout) << name << ' ' << dat << ' ';
  } else {
    (*strout) << dat << ' ';
  }
  flush();
  return 0;
}

int text_out_file::double_out(double dat, std::string name) {
  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::double_out()",
		  gsl_efailed);
  }
  if (!finite(dat)) dat=0.0;
  if (name.length()>0) {
    (*strout) << name << ' ' << dat << ' ';
  } else {
    (*strout) << dat << ' ';
  }
  flush();
  return 0;
}

int text_out_file::float_out(float dat, std::string name) {
  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::float_out()",
		  gsl_efailed);
  }
  if (!finite(dat)) dat=0.0;
  if (name.length()>0) {
    (*strout) << name << ' ' << dat << ' ';
  } else {
    (*strout) << dat << ' ';
  }
  flush();
  return 0;
}

int text_out_file::int_out(int dat, std::string name) {
  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::int_out()",
		  gsl_efailed);
  }
  if (name.length()>0) {
    (*strout) << name << ' ' << dat << ' ';
  } else {
    (*strout) << dat << ' ';
  }
  flush();
  return 0;
}

int text_out_file::long_out(unsigned long int dat, std::string name) {
  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::long_out()",
		  gsl_efailed);
  }
  if (name.length()>0) {
    (*strout) << name << ' ' << dat << ' ';
  } else {
    (*strout) << dat << ' ';
  }
  flush();
  return 0;
}

std::string text_out_file::reformat_string(std::string in) {
  for(int i=0;i<((int)in.length());i++) {
    if (in[i]==13) in[i]=32;
    if (extra_brackets && (in[i]=='<' || in[i]=='>')) {
      char ch=in[i];
      int ix=i+1;
      while(ix<((int)in.length()) && in[ix]==ch) ix++;
      in=in.substr(0,i+1)+ch+in.substr(ix,in.length()-ix);
      i=ix;
    } 
  }
  return in;
}

int text_out_file::string_out(string dat, std::string name) {

  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::string_out()",
		  gsl_efailed);
  }

  /// Add brackets if necessary
  std::string dnew=reformat_string(dat);

  /// Output the string
  end_line();
  if (name.length()>0) {
    (*outs) << name << ' ' << dat << endl;
  } else {
    (*outs) << dat << endl;
  }
  end_line();
  return 0;
}

int text_out_file::word_out(string dat, std::string name) {

  if (strout==0) {
    O2SCL_ERR_RET("No file open in text_out_file::word_out()",
		  gsl_efailed);
  }

  /// Add brackets and change spaces to underscores if necessary
  for(int i=0;i<((int)dat.length());i++) {
    if (dat[i]==' ' || (dat[i]>=9 && dat[i]<=13)) dat[i]='_';
    if (extra_brackets && (dat[i]=='<' || dat[i]=='>')) {
      char ch=dat[i];
      // Find the end of the <'s or >'s
      int ix=i+1;
      while(ix<((int)dat.length()) && dat[ix]==ch) {
	ix++;
      }
      // Add an extra < or > and adjust i accordingly
      dat=dat.substr(0,i+1)+ch+dat.substr(ix,dat.length()-ix);
      i=ix;
    } 
  }
  /// Output the string
  if (name.length()>0) {
    (*strout) << name << ' ' << dat << ' ';
  } else {
    (*strout) << dat << ' ';
  }
  flush();
  return 0;
}

int text_out_file::comment_out(std::string dat) {
  end_line();
  (*outs) << "# " << dat << endl;
  end_line();
  return 0;
}

int text_out_file::flush() {
  int loc;
  string s=strout->str();

  if (((int)s.length())>=file_width) {

    loc=s.find_last_of(' ',file_width);
    if (loc==((int)string::npos)) {
      loc=(file_width-1);
    }
    string t=s.substr(0,loc);

    (*outs) << t << endl;
    
    // Delete the old, and make a new ostrstream
    // and set the flags right.
    delete strout;
    strout=new ostringstream;
    
    strout->precision(props->precision());
    strout->fill(props->fill());
    strout->flags(props->flags());
    
    // Add the subset of s that wasn't output already.
    t=s.substr(loc+1,s.length()-loc-1);
    *strout << t;
  }

  return 0;
}

int text_out_file::end_object() {
  if (hcs.top()==false) {
    char_out_internal('>');
  }
  hcs.pop();
  if (hcs.size()==0) end_line();
  return 0;
}

bool text_out_file::is_hc_type(std::string type) {
  if (type!="bool" && type!="char" && type!="double" && type!="int" &&
      type!="string" && type!="word" && type!="bool[]" && type!="char[]" && 
      type!="double[]" && type!="int[]" && type!="string[]" && 
      type!="word[]" && type!="bool[][]" && type!="char[][]" && 
      type!="double[][]" && type!="int[][]" && type!="string[][]" && 
      type!="word[][]" && type!="float" && type!="float[]" && 
      type!="float[][]") {
    return false;
  }
  return true;
}

int text_out_file::start_object(std::string type, std::string name) {
  word_out(type);
  if (!is_hc_type(type)) {
    char_out_internal('<');
    hcs.push(false);
  } else {
    hcs.push(true);
  }
  if (name.length()>0) word_out(name);
  return 0;
} 

// ------------------------------------------------------------
// text_in_file functions
// ------------------------------------------------------------

bool text_in_file::is_hc_type(std::string type) {
  if (type!="bool" && type!="char" && type!="double" && type!="int" &&
      type!="string" && type!="word" && type!="bool[]" && type!="char[]" && 
      type!="double[]" && type!="int[]" && type!="string[]" && 
      type!="word[]" && type!="bool[][]" && type!="char[][]" && 
      type!="double[][]" && type!="int[][]" && type!="string[][]" && 
      type!="word[][]" && type!="float" && type!="float[]" && 
      type!="float[][]") {
    return false;
  }
  return true;
}

int text_in_file::init_file() {
  return 0;
}

text_in_file::text_in_file(std::string file_name) {
  ifstream *fin=new ifstream;
  fin->open(file_name.c_str());
  ins=fin;
  from_string=true;
}

text_in_file::text_in_file(istream *in_file) {
  ins=in_file;
  from_string=false;
}

text_in_file::~text_in_file() {
  if (from_string==true) {
    ifstream *fin=(ifstream *)ins;
    fin->close();
    delete fin;
  }
}

int text_in_file::bool_in(bool &dat, std::string name) {
  string tmp;
  if (name.length()>0) {
    if (!((*ins) >> tmp)) {
      O2SCL_ERR_RET("Input name in text_in_file::bool_in() failed.",
		    gsl_efailed);
    }
  }
  word_in(tmp);
  if (tmp.length()>0) {
    if (tmp[0]=='t' || tmp[0]=='T' ||
	tmp[0]=='1') {
      dat=true;
    } else {
      dat=false;
    }
    return 0;
  }
  O2SCL_ERR_RET("Input bool in text_in_file::bool_in() failed.",
		gsl_efailed);
}

int text_in_file::char_in(char &dat, std::string name) {
  string tmp;
  if (name.length()>0) {
    if (!((*ins) >> tmp)) {
      O2SCL_ERR_RET("Input name in text_in_file::char_in() failed.",
		    gsl_efailed);
    }
  }
  if ((*ins) >> tmp) {
    dat=tmp[0];
    return 0;
  }
  O2SCL_ERR_RET("Input character in text_in_file::char_in() failed.",
		gsl_efailed);
}

int text_in_file::double_in(double &dat, std::string name) {
  string tmp;
  if (name.length()>0) {
    if (!((*ins) >> tmp)) {
      O2SCL_ERR_RET("Input name in text_in_file::double_in() failed.",
		    gsl_efailed);
    }
  }
  if ((*ins) >> dat) {
    return 0;
  }
  O2SCL_ERR_RET("Input double in text_in_file::double_in() failed.",
		gsl_efailed);
}

int text_in_file::float_in(float &dat, std::string name) {
  string tmp;
  if (name.length()>0) {
    if (!((*ins) >> tmp)) {
      O2SCL_ERR_RET("Input name in text_in_file::float_in() failed.",
		    gsl_efailed);
    }
  }
  if ((*ins) >> dat) {
    return 0;
  }
  O2SCL_ERR_RET("Input float in text_in_file::float_in() failed.",
		gsl_efailed);
}

int text_in_file::int_in(int &dat, std::string name) {
  string tmp;
  if (name.length()>0) {
    if (!((*ins) >> tmp)) {
      O2SCL_ERR_RET("Input name in text_in_file::int_in() failed.",
		    gsl_efailed);
    }
  }
  if ((*ins) >> dat) {
    return 0;
  }
  O2SCL_ERR_RET("Input integer in text_in_file::int_in() failed.",
		gsl_efailed);
}

int text_in_file::long_in(unsigned long int &dat, std::string name) {
  string tmp;
  if (name.length()>0) {
    if (!((*ins) >> tmp)) {
      O2SCL_ERR_RET("Input name in text_in_file::long_in() failed.",
		    gsl_efailed);
    }
  }
  if ((*ins) >> dat) {
    return 0;
  }
  O2SCL_ERR_RET("Input integer in text_in_file::long_in() failed.",
		gsl_efailed);
}

std::string text_in_file::reformat_string(string dat) {
  std::string dnew;
  size_t ix=0;
  while(ix<dat.length()) {
    size_t left=dat.find('<');
    if (left<dat.length()-1) {
      size_t right=left;
      while(right<dat.length()-1 && dat[right+1]=='<') right++;
      dnew=dat.substr(0,left-1)+dat.substr(right,dat.length()-right);
    }
  }
  return dat;
}

int text_in_file::string_in(string &dat, std::string name) {
  string tmp;
  if (name.length()>0) {
    if (!((*ins) >> tmp)) {
      O2SCL_ERR_RET("Input name in text_in_file::string_in() failed.",
		    gsl_efailed);
    }
  }
  getline(*ins,dat);
  if (o2scl::count_words(dat)==0) {
    getline(*ins,dat);
  }
  return 0;
}

int text_in_file::word_in(string &dat, std::string name) {
  string tmp;
  if (name.length()>0) {
    if (!((*ins) >> tmp)) {
      O2SCL_ERR_RET("Input name in text_in_file::word_in() failed.",
		    gsl_efailed);
    }
  }
  if ((*ins) >> dat) {
    return 0;
  }
  O2SCL_ERR_RET("Input word failed in text_in_file::text_in_file::word_in().",
		gsl_efailed);
}

int text_in_file::start_object(std::string &type, std::string &name) {
  if (word_in_noerr(type)!=0) {
    // We don't call the error handler here, because this 
    // happens at the end of a file, which is normal.
    return gsl_efailed;
  }
  while (type[0]=='#') {
    string line;
    getline(*ins,line);
    if (word_in(type)!=0) {
      O2SCL_ERR_RET("Input type failed in text_in_file::read_type().",
		    gsl_efailed);
    }
  }
  if (!is_hc_type(type)) {
    char ch;
    char_in(ch);
    hcs.push(false);
  } else {
    hcs.push(true);
  }
  if (name.length()>0) {
    if (word_in(name)!=0) {
      O2SCL_ERR_RET("Input name failed in text_in_file::read_type().",
		    gsl_efailed);
    }
  }
  return 0;
}

int text_in_file::skip_object() {
  std::string tmp;
  do {
    word_in(tmp);
  } while (tmp!=">");
  return 0;
}

int text_in_file::end_object() {
  if (hcs.top()==false) {
    char ch;
    char_in(ch);
  }
  hcs.pop();
  return 0;
}

