/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/array.h>
#include <o2scl/vector.h>
#include <o2scl/columnify.h>
#include <o2scl/test_mgr.h>
#include <o2scl/permutation.h>
#include <climits>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  
  // Test vector_sort
  double st1[5]={3.1,4.1,5.9,2.6,5.3};
  vector_sort<double[5],double>(5,st1);
  vector_out(cout,5,st1,true);
  t.test_rel(st1[0],2.6,1.0e-10,"sort 1");
  t.test_rel(st1[1],3.1,1.0e-10,"sort 1");
  t.test_rel(st1[2],4.1,1.0e-10,"sort 1");
  t.test_rel(st1[3],5.3,1.0e-10,"sort 1");
  t.test_rel(st1[4],5.9,1.0e-10,"sort 1");

  // Test vector_sort_double
  double *st2=new double[5];
  st2[0]=3.1;
  st2[1]=4.1;
  st2[2]=5.9;
  st2[3]=2.6;
  st2[4]=5.3;
  vector_sort_double(5,st2);
  vector_out(cout,5,st2,true);
  t.test_rel(st2[0],2.6,1.0e-10,"sort 2");
  t.test_rel(st2[1],3.1,1.0e-10,"sort 2");
  t.test_rel(st2[2],4.1,1.0e-10,"sort 2");
  t.test_rel(st2[3],5.3,1.0e-10,"sort 2");
  t.test_rel(st2[4],5.9,1.0e-10,"sort 2");
  delete[] st2;

  // Test vector_sort_index
  double *st3=new double[5];
  size_t ind[5];
  st3[0]=3.1;
  st3[1]=4.1;
  st3[2]=5.9;
  st3[3]=2.6;
  st3[4]=5.3;
  vector_sort_index(5,st3,ind);
  vector_out(cout,5,st3,true);
  vector_out(cout,5,ind,true);
  t.test_gen(ind[0]==3,"sort 3");
  t.test_gen(ind[1]==0,"sort 3");
  t.test_gen(ind[2]==1,"sort 3");
  t.test_gen(ind[3]==4,"sort 3");
  t.test_gen(ind[4]==2,"sort 3");

  // Show that it works with a permutation
  double *st4=new double[5];
  permutation p(5);
  st4[0]=3.1;
  st4[1]=4.1;
  st4[2]=5.9;
  st4[3]=2.6;
  st4[4]=5.3;
  vector_sort_index(5,st4,p);
  vector_out(cout,5,st4,true);
  vector_out(cout,5,p,true);
  t.test_gen(p[0]==3,"sort 3");
  t.test_gen(p[1]==0,"sort 3");
  t.test_gen(p[2]==1,"sort 3");
  t.test_gen(p[3]==4,"sort 3");
  t.test_gen(p[4]==2,"sort 3");

  // Compute the inverse
  permutation p2=p.inverse();
  vector_out(cout,5,p2,true);

  t.report();
  return 0;
}

