/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <o2scl/funct.h>
#include <o2scl/gsl_inte_qawc.h>
#include <o2scl/cern_cauchy.h>

using namespace std;
using namespace o2scl;

double testfun(double tx, int &pa) {
  return exp(-tx);
}

double testfun2(double tx, int &pa) {
  return log(tx+1.0);
}

double testfun3(double tx, int &pa) {
  return log(tx+1.0)/(tx-1.0);
}

funct_fptr_noerr<int > tf(testfun);
funct_fptr_noerr<int > tf2(testfun2);
funct_fptr_noerr<int > tf3(testfun3);

int main(void) {
  double a=3.0, calc, exact, diff;
  gsl_inte_qawc<int ,funct_fptr_noerr<int > > cc;
  cern_cauchy<int ,funct_fptr_noerr<int > > cc3;
  test_mgr t;
  t.set_output_level(2);
  int vp=0;

  cout.setf(ios::scientific);
  cout.precision(10);
  
  cc.s=2.0;
  calc=cc.integ(tf,1.0,3.0,vp);
  t.test_rel(calc,-0.286166,1.0e-5,"gsl_inte_qawc 1");
  
  cc.s=1.0;
  calc=cc.integ(tf2,0.0,2.0,vp);
  t.test_rel(calc,1.03065,1.0e-5,"gsl_inte_qawc 2");

  cc3.s=1.0;
  calc=cc3.integ(tf3,0.0,2.0,vp);
  t.test_rel(calc,1.03065,1.0e-5,"cern_cauchy 2");
  
  t.report();
  return 0;
}

