/*
  -------------------------------------------------------------------
  
  Copyright (C) 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
/*
  Just a short program to analyze and summarize O2scl test results.
*/
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <cstdio>

using namespace std;

int main(void) {

  ifstream fin("testlist");
  if (!fin) {
    cerr << "Could not open list of files to test." << endl;
    exit(-1);
  }
  vector<string> fns;
  
  while(!fin.eof()) {
    string tmp;
    fin >> tmp;
    if (tmp.length()>0) {
      fns.push_back(tmp);
    }
  }

  bool failed=false;

  const size_t nch=300;
  vector<string>::iterator it;
  for(it=fns.begin();it!=fns.end();it++) {
    FILE *fp=fopen(it->c_str(),"r");
    if (fp==0) {
      cerr << "Failed to open test file: " << (*it) << endl;
      failed=true;
    } else {
      fseek(fp,-nch,SEEK_END);
      vector<string> lines;
      while (!feof(fp)) {
	char ctmp[nch];
	fgets(ctmp,nch,fp);
	if (ctmp) {
	  lines.push_back(ctmp);
	}
      }
      if (lines.size()>=3) {
	if (lines[lines.size()-1]!="All tests passed.\n") {
	  cerr << "Some tests failed in file: " << (*it) << endl;
	  failed=true;
	}
      } else {
	cerr << "Some tests failed in file: " << (*it) << endl;
	failed=true;
      }
      fclose(fp);
    }
  }

  if (failed==true) {
    cout << "One or more O2scl tests failed." << endl;
    exit(-1);
  }

  cout << "All O2scl tests passed." << endl;

  return 0;
}
