/*
  -------------------------------------------------------------------

  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner

  This file is part of O2scl.

  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.

  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>

#include <gsl/gsl_blas.h>
#include <gsl/gsl_linalg.h>

#include <o2scl/qr.h>
#include <o2scl/test_mgr.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/omatrix_tlate.h>
#include <o2scl/columnify.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);

  {
    using namespace o2scl_cblas;
    using namespace o2scl_linalg;

    omatrix gm1(5,5), om1(5,5), gm2(5,5), om2(5,5), gm3(5,5), om3(5,5);
    ovector gv1(5), ov1(5), gv2(5), ov2(5), gv3(5), ov3(5);
    permutation gp1(5), op1(5), mp1(5);
    double mm1[5][5], d, mv1[5], mv2[5], mm2[5][5], mv3[5], mm3[5][5];
    int sig, ret;

    for(size_t ik=0;ik<10;ik++) {

      // Test decomp

      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(((double)(i)));
	ov1[i]=cos(((double)(i)));
	mv1[i]=cos(((double)(i)));
	gv2[i]=cos(((double)(i)));
	ov2[i]=cos(((double)(i)));
	mv2[i]=cos(((double)(i)));
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  om1[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  gm2[i][j]=((double)(ik+1))/(1.0+i+j);
	  om2[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm2[i][j]=((double)(ik+1))/(1.0+i+j);
	}
      }

      gsl_linalg_QR_decomp(&gm1,&gv1);
      QR_decomp(5,5,om1,ov1);
      QR_decomp(5,5,mm1,mv1);
      t.test_rel_mat(5,5,gm1,om1,3.0e-12,"qr decomp 1");
      t.test_rel_arr(5,gv1,ov1,3.0e-12,"qr decomp 2");
      t.test_rel_mat(5,5,gm1,mm1,3.0e-12,"qr decomp 3");
      t.test_rel_arr(5,gv1,mv1,3.0e-12,"qr decomp 4");

      // Test QTvec

      ret=gsl_linalg_QR_QTvec(&gm1,&gv1,&gv2);
      ret=QR_QTvec(5,5,om1,ov1,ov2);
      ret=QR_QTvec(5,5,mm1,mv1,mv2);
      t.test_rel_arr(5,gv2,ov2,3.0e-12,"qr qtvec 1");
      t.test_rel_arr(5,gv2,mv2,3.0e-12,"qr qtvec 2");

      // Test solve

      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(((double)(i)));
	ov1[i]=cos(((double)(i)));
	mv1[i]=cos(((double)(i)));
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  om1[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  gm2[i][j]=((double)(ik+1))/(1.0+i+j);
	  om2[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm2[i][j]=((double)(ik+1))/(1.0+i+j);
	}
      }

      gsl_linalg_QR_decomp(&gm1,&gv2);
      gsl_linalg_QR_solve(&gm1,&gv2,&gv1,&gv3);

      QR_decomp(5,5,om1,ov2);
      QR_solve(5,om1,ov2,ov1,ov3);

      QR_decomp(5,5,mm1,mv2);
      QR_solve(5,mm1,mv2,mv1,mv3);

      t.test_rel_arr(5,gv3,ov3,3.0e-12,"qr solve 1");
      t.test_rel_arr(5,gv3,mv3,3.0e-12,"qr solve 2");

      // Test unpack

      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(((double)(i)));
	ov1[i]=cos(((double)(i)));
	mv1[i]=cos(((double)(i)));
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  om1[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  gm2[i][j]=((double)(ik+1))/(1.0+i+j);
	  om2[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm2[i][j]=((double)(ik+1))/(1.0+i+j);
	}
      }
    
      gsl_linalg_QR_decomp(&gm1,&gv2);
      gsl_linalg_QR_unpack(&gm1,&gv2,&gm2,&gm3);

      QR_decomp(5,5,om1,ov2);
      QR_unpack(5,5,om1,ov2,om2,om3);

      QR_decomp(5,5,mm1,mv2);
      QR_unpack(5,5,mm1,mv2,mm2,mm3);

      t.test_rel_mat(5,5,gm2,om2,3.0e-12,"qr decomp 1");
      t.test_rel_mat(5,5,gm3,om3,3.0e-12,"qr decomp 2");
      t.test_rel_mat(5,5,gm2,mm2,3.0e-12,"qr decomp 3");
      t.test_rel_mat(5,5,gm3,mm3,3.0e-12,"qr decomp 4");

    }

  }
  {
    using namespace o2scl_cblas_paren;
    using namespace o2scl_linalg_paren;

    omatrix gm1(5,5), om1(5,5), gm2(5,5), om2(5,5), gm3(5,5), om3(5,5);
    ovector gv1(5), ov1(5), gv2(5), ov2(5), gv3(5), ov3(5);
    permutation gp1(5), op1(5), mp1(5);
    double mm1[5][5], d, mv1[5], mv2[5], mm2[5][5], mv3[5], mm3[5][5];
    int sig, ret;

    for(size_t ik=0;ik<10;ik++) {

      // Test decomp

      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(((double)(i)));
	ov1[i]=cos(((double)(i)));
	mv1[i]=cos(((double)(i)));
	gv2[i]=cos(((double)(i)));
	ov2[i]=cos(((double)(i)));
	mv2[i]=cos(((double)(i)));
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  om1[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  gm2[i][j]=((double)(ik+1))/(1.0+i+j);
	  om2[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm2[i][j]=((double)(ik+1))/(1.0+i+j);
	}
      }

      gsl_linalg_QR_decomp(&gm1,&gv1);
      QR_decomp(5,5,om1,ov1);
      t.test_rel_mat(5,5,gm1,om1,3.0e-12,"qr decomp 1");
      t.test_rel_arr(5,gv1,ov1,3.0e-12,"qr decomp 2");

      // Test QTvec

      ret=gsl_linalg_QR_QTvec(&gm1,&gv1,&gv2);
      ret=QR_QTvec(5,5,om1,ov1,ov2);
      t.test_rel_arr(5,gv2,ov2,3.0e-12,"qr qtvec 1");

      // Test solve

      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(((double)(i)));
	ov1[i]=cos(((double)(i)));
	mv1[i]=cos(((double)(i)));
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  om1[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  gm2[i][j]=((double)(ik+1))/(1.0+i+j);
	  om2[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm2[i][j]=((double)(ik+1))/(1.0+i+j);
	}
      }

      gsl_linalg_QR_decomp(&gm1,&gv2);
      gsl_linalg_QR_solve(&gm1,&gv2,&gv1,&gv3);

      QR_decomp(5,5,om1,ov2);
      QR_solve(5,om1,ov2,ov1,ov3);

      t.test_rel_arr(5,gv3,ov3,3.0e-12,"qr solve 1");

      // Test unpack

      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(((double)(i)));
	ov1[i]=cos(((double)(i)));
	mv1[i]=cos(((double)(i)));
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  om1[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm1[i][j]=((double)(ik+1))/(1.0+i+j);
	  gm2[i][j]=((double)(ik+1))/(1.0+i+j);
	  om2[i][j]=((double)(ik+1))/(1.0+i+j);
	  mm2[i][j]=((double)(ik+1))/(1.0+i+j);
	}
      }
    
      gsl_linalg_QR_decomp(&gm1,&gv2);
      gsl_linalg_QR_unpack(&gm1,&gv2,&gm2,&gm3);

      QR_decomp(5,5,om1,ov2);
      QR_unpack(5,5,om1,ov2,om2,om3);

      t.test_rel_mat(5,5,gm2,om2,3.0e-12,"qr decomp 1");
      t.test_rel_mat(5,5,gm3,om3,3.0e-12,"qr decomp 2");

    }

  }

  t.report();

  return 0;
}

