/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/omatrix_tlate.h>
#include <o2scl/ode_iv_solve.h>
#include <o2scl/test_mgr.h>
#include <o2scl/ode_funct.h>

using namespace std;
using namespace o2scl;

int derivs(double x, size_t nv, const ovector_base &y, ovector_base &dydx, 
	   int &pa) {
  dydx[0]=y[0];
  return 0;
}

int derivsa(double x, size_t nv, const double y[1], double dydx[1], 
	    int &pa) {
  dydx[0]=y[0];
  return 0;
}

int main(void) {
  int i;
  ovector y, dydx, yout, yerr;
  ode_funct_fptr<int> od(derivs);
  ode_iv_solve<int ,ode_funct<int>,ovector_base,ovector,
    ovector_alloc> ivs;
  ode_vfunct_fptr<int ,1> oda(derivsa);
  ode_iv_solve<int ,ode_vfunct<int ,1>,double[1],double[1],
    array_alloc<double[1]> > ivsa;
  int vp=0;
  test_mgr t;
  double ya[1], dydxa[1], youta[1], yerra[1];

  y.allocate(1);
  dydx.allocate(1);
  yout.allocate(1);
  yerr.allocate(1);

  t.set_output_level(1);

  cout.setf(ios::scientific);

  // ------------------------------------------------

  cout << endl;
  ovector yend(1);
  
  y[0]=1.0;
  cout << "Final value 1: " << endl;
  ivs.solve_final_value(1.0,5.0,4.0,1,y,yend,vp,od);
  
  cout << yend[0] << " " << fabs(yend[0]-exp(4.0))/exp(4.0) << endl;
  t.test_rel(yend[0],exp(4.0),1.0e-6,"final value");

  cout << endl;
  
  // ------------------------------------------------

  double yenda[1];

  ya[0]=1.0;
  cout << "Final value 2: " << endl;
  ivsa.solve_final_value(1.0,5.0,4.0,1,ya,yenda,vp,oda);
  
  cout << yenda[0] << " " << fabs(yenda[0]-exp(4.0))/exp(4.0) << endl;
  t.test_rel(yenda[0],exp(4.0),1.0e-6,"final value");

  cout << endl;

  // ------------------------------------------------
  
  int ngrid=11;
  ovector xgrid(11);
  omatrix ygrid(11,1);
  omatrix dydxgrid(11,1);
  omatrix err_grid(11,1);
  y[0]=1.0;
  
  cout << "Grid_derivs 1: " << endl;
  ivs.solve_grid<omatrix,omatrix_row,omatrix_alloc>
    (1.0,5.0,4.0,1,y,ngrid,xgrid,ygrid,dydxgrid,vp,od);
  
  for(i=0;i<ngrid;i++) {
    cout << xgrid[i] << " " << ygrid[i][0] << " " << dydxgrid[i][0] << " " 
	 << exp(xgrid[i]-1.0) << " "
	 << fabs(ygrid[i][0]-exp(xgrid[i]-1.0))/exp(xgrid[i]-1.0) 
	 << endl;
    t.test_rel(ygrid[i][0],exp(xgrid[i]-1.0),1.0e-6,"grid");
  }

  xgrid.free();
  ygrid.free();
  dydxgrid.free();
  
  cout << endl;

  // ------------------------------------------------
  // Test solve_table
  
  size_t ng3=15;
  ovector cgrid(ng3);
  omatrix dgrid(ng3,1);
  omatrix egrid(ng3,1);
  omatrix fgrid(ng3,1);
  y[0]=1.0;
  
  cout << "ivsn: " << ivs.nsteps << endl;

  ivs.verbose=1;
  cout << "Table: " << endl;
  ivs.solve_table<omatrix,omatrix_row>
    (1.0,10.0,0.4,1,y,ng3,cgrid,dgrid,egrid,fgrid,vp,od);
  cout << endl;
  
  cout << "Table results: " << endl;
  for(size_t k=0;k<ng3;k++) {
    cout << k << " " << cgrid[k] << " " << dgrid[k][0] << " " 
	 << egrid[k][0] << " "
	 << fgrid[k][0] << endl;
    t.test_rel(dgrid[k][0],exp(cgrid[k]-1.0),1.0e-6,"grid");
  }
  size_t ns1=ivs.nsteps;

  // ------------------------------------------------
  // Compare results from solve_table() to those from solve_final_value().

  y[0]=1.0;
  ivs.solve_final_value(1.0,10.0,0.4,1,y,yend,vp,od);
  t.test_rel(dgrid[ng3-1][0],yend[0],1.0e-14,"sync");
  t.test_gen(ivs.nsteps==ns1,"sync 2");

  // ------------------------------------------------

  t.report();
  return 0;
}

