/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>
#include <o2scl/test_mgr.h>
#include <o2scl/convert_units.h>

using namespace o2scl;
using namespace std;

int main(void) {
  cout.setf(ios::scientific);
  test_mgr t;
  t.set_output_level(1);

  // Make several conversions to get back to unity
  
  {
    convert_units cu;
    cu.energy_conv();
    double x=cu.convert("J","K",1.0);
    double x2=cu.convert("K","m^-1",x);
    double x3=cu.convert("m^-1","s^-1",x2);
    double x4=cu.convert("s^-1","fm^-1",x3);
    double x5=cu.convert("fm^-1","eV",x4);
    double x6=cu.convert("eV","J",x5);
    t.test_rel(x6,1.0,1.0e-7,"convert7");
  }

  // Test all the permutations of mixing two conversions
  // to generate a third

  //1 
  {
    convert_units cu4;
    cu4.insert_cache("in","ft",1.0/12.0);
    cu4.insert_cache("in","cm",2.54);

    double res=cu4.convert("ft","cm",1.0);
    t.test_rel(res,12.0*2.540,1.0e-10,"1");
    res=cu4.convert("cm","ft",1.0);
    t.test_rel(res,1.0/(12.0*2.540),1.0e-10,"1");
  }

  //2 
  {
    convert_units cu4;
    cu4.insert_cache("ft","in",12.0);
    cu4.insert_cache("in","cm",2.54);

    double res=cu4.convert("ft","cm",1.0);
    t.test_rel(res,12.0*2.540,1.0e-10,"1");
    res=cu4.convert("cm","ft",1.0);
    t.test_rel(res,1.0/(12.0*2.540),1.0e-10,"1");
  }

  //3 
  {
    convert_units cu4;
    cu4.insert_cache("in","ft",1.0/12.0);
    cu4.insert_cache("cm","in",1.0/2.54);

    double res=cu4.convert("ft","cm",1.0);
    t.test_rel(res,12.0*2.540,1.0e-10,"1");
    res=cu4.convert("cm","ft",1.0);
    t.test_rel(res,1.0/(12.0*2.540),1.0e-10,"1");
  }

  //4
  {
    convert_units cu4;
    cu4.insert_cache("ft","in",12.0);
    cu4.insert_cache("cm","in",1.0/2.54);

    double res=cu4.convert("ft","cm",1.0);
    t.test_rel(res,12.0*2.540,1.0e-10,"1");
    res=cu4.convert("cm","ft",1.0);
    t.test_rel(res,1.0/(12.0*2.540),1.0e-10,"1");
  }

  //5
  {
    convert_units cu4;
    cu4.insert_cache("in","cm",2.54);
    cu4.insert_cache("in","ft",1.0/12.0);

    double res=cu4.convert("ft","cm",1.0);
    t.test_rel(res,12.0*2.540,1.0e-10,"1");
    res=cu4.convert("cm","ft",1.0);
    t.test_rel(res,1.0/(12.0*2.540),1.0e-10,"1");
  }

  //6
  {
    convert_units cu4;
    cu4.insert_cache("in","cm",2.54);
    cu4.insert_cache("ft","in",12.0);

    double res=cu4.convert("ft","cm",1.0);
    t.test_rel(res,12.0*2.540,1.0e-10,"1");
    res=cu4.convert("cm","ft",1.0);
    t.test_rel(res,1.0/(12.0*2.540),1.0e-10,"1");
  }

  //7
  {
    convert_units cu4;
    cu4.insert_cache("cm","in",1.0/2.54);
    cu4.insert_cache("in","ft",1.0/12.0);

    double res=cu4.convert("ft","cm",1.0);
    t.test_rel(res,12.0*2.540,1.0e-10,"1");
    res=cu4.convert("cm","ft",1.0);
    t.test_rel(res,1.0/(12.0*2.540),1.0e-10,"1");
  }

  //8
  {
    convert_units cu4;
    cu4.insert_cache("cm","in",1.0/2.54);
    cu4.insert_cache("ft","in",12.0);

    double res=cu4.convert("ft","cm",1.0);
    t.test_rel(res,12.0*2.540,1.0e-10,"1");
    res=cu4.convert("cm","ft",1.0);
    t.test_rel(res,1.0/(12.0*2.540),1.0e-10,"1");
  }

  // Test converting back and forth between several units

  { 
    convert_units cu5;
    cu5.energy_conv();
    
    t.test_rel(cu5.convert("J","eV",cu5.convert("eV","J",1.0)),
	       1.0,1.0e-10,"1");
    t.test_rel(cu5.convert("J","erg",cu5.convert("erg","J",1.0)),
	       1.0,1.0e-10,"1");
    t.test_rel(cu5.convert("J","m^-1",cu5.convert("m^-1","J",1.0)),
	       1.0,1.0e-10,"1");
    t.test_rel(cu5.convert("J","s^-1",cu5.convert("s^-1","J",1.0)),
	       1.0,1.0e-10,"1");
    t.test_rel(cu5.convert("J","K",cu5.convert("K","J",1.0)),
	       1.0,1.0e-10,"1");
  }

  t.report();
  return 0;
}
