/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_CLASSICAL_H
#define O2SCL_CLASSICAL_H

#include <string>
#include <iostream>
#include <fstream>
#include <cmath>
#include <o2scl/constants.h>
#include <o2scl/mroot.h>
#include <o2scl/inte.h>
#include <o2scl/part.h>
#include <o2scl/fermion.h>
#include <o2scl/boson.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Classical particle class
      
      \future Write a calc_density_zerot() function for 
      completeness?
  */
  class classical : public part {

  public:

    /** \brief Create a classical particle with mass \c m  
	and degeneracy \c g 
    */
    classical(double m=0.0, double g=0.0);

    /** 
	\brief Calculate properties as function of chemical potential
     */
    virtual int calc_mu(const double temper);

    /** \brief Calculate properties as function of density
     */
    virtual int calc_density(const double temper);

    /// Return string denoting type ("classical")
    virtual const char *type() { return "classical"; }
    
  };

#ifndef DOXYGENP
}
#endif

#endif
