/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#ifndef O2SCL_GSL_MROOT_HYBRIDS_B_H
#define O2SCL_GSL_MROOT_HYBRIDS_B_H

#include <cmath>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Base functionality for the GSL hybrids solver used 
      in \ref gsl_mroot_hybrids
      
      This is a trivial recasting of the functions that were in file
      scope in the GSL version of the hybrids solver. 
  */
  class mroot_hybrids_base {
  protected:

    /** \brief Compute the Euclidean norm of \c f

	\future Replace this with \c dnrm2 from \ref cblas_base.h
     */
    double enorm(const gsl_vector *f);
    
    /** \brief Compute the norm of the vector \f$ \vec{v} \f$ defined
	by \f$ v_i = d_i f_i \f$
    */
    double scaled_enorm(const gsl_vector *d, const gsl_vector *f);

    /** \brief Compute the norm of the vector \f$ \vec{v} \f$ defined
	by \f$ v_i = a_i + b_i \f$
    */
    double enorm_sum(const gsl_vector *a, const gsl_vector *b);
  
    /// Compute the actual reduction
    double compute_actual_reduction(double fnorm, double fnorm1);

    /// Compute the predicted reduction phi1p=|Q^T f + R dx| 
    double compute_predicted_reduction(double fnorm, double fnorm1);
    
    /** \brief Compute \f$ Q^{T} f \f$
	
	\future This is just right-multiplication, so we could
	use the \o2 cblas routines instead
     */
    void compute_qtf(const gsl_matrix *q, const gsl_vector *f, 
		     gsl_vector *qtf);
    
    /// Compute the Gauss-Newton direction
    int newton_direction(const gsl_matrix *r, const gsl_vector *qtf, 
			 gsl_vector *p);

    /// Compute the gradient direction
    void gradient_direction(const gsl_matrix *r, const gsl_vector *qtf,
			    const gsl_vector *diag, gsl_vector *g);

    /// Compute the point at which the gradient is minimized
    void minimum_step(double gnorm, const gsl_vector *diag, gsl_vector *g);


  };
  
#ifndef DOXYGENP
}
#endif

#endif
